@@ -1,10 +1,7 @@
-from typing import List, Literal, Optional, Sequence, Tuple, Union
+from functools import partial
+from typing import List, Literal, Optional, Sequence, Union
 
-from astropy.coordinates import (
-    get_body,
-    HeliocentricMeanEcliptic,
-    solar_system_ephemeris,
-)
+from astropy.coordinates import solar_system_ephemeris
 import astropy.units as u
 from astropy.units import Quantity, quantity_input
 from astropy.time import Time
@@ -12,25 +9,25 @@ import healpy as hp
 import numpy as np
 from numpy.typing import NDArray
 
-from zodipy._integral import trapezoidal
-from zodipy._interp import interp_phase_coeffs, interp_comp_spectral_params
-from zodipy._los_config import integration_config_registry
-from zodipy._model import InterplanetaryDustModel
+from zodipy._emission import get_emission_step
+from zodipy._ephemeris import get_earth_position, get_observer_position
+from zodipy._integral import trapezoidal_regular_grid
+from zodipy._line_of_sight import get_line_of_sight
+from zodipy._unit_vector import (
+    get_unit_vector_from_angles,
+    get_unit_vector_from_pixels,
+)
 from zodipy.models import model_registry
 
 
 __all__ = ("Zodipy",)
 
-# L2 not included in the astropy api so we manually include support for it.
-ADDITIONAL_SUPPORTED_OBS = ["semb-l2"]
-DISTANCE_EARTH_TO_L2 = 0.009896235034000056 * u.AU
-
 
 class Zodipy:
     """The Zodipy interface.
 
     Zodipy simulates the Zodiacal emission that a Solar System observer is
-    predicted to see given the Kelsall et al. (1998) interplanetary dust model. 
+    predicted to see given the Kelsall et al. (1998) interplanetary dust model.
     """
 
     def __init__(self, model: str = "DIRBE", ephemeris: str = "de432s") -> None:
@@ -43,38 +40,27 @@ class Zodipy:
             available models with `zodipy.MODELS`.
         ephemeris
             Ephemeris used to compute the positions of the observer and Earth.
-            Defaults to 'de432s' which requires downloading (and caching) a 10 
+            Defaults to 'de432s' which requires downloading (and caching) a 10
             MB file. For more information on available ephemeridis, please visit
             https://docs.astropy.org/en/stable/coordinates/solarsystem.html
         """
 
-        self._model = model_registry.get_model(model)
-        self._line_of_sights = integration_config_registry.get_config()
+        self.model = model_registry.get_model(model)
         solar_system_ephemeris.set(ephemeris)
 
     @property
-    def model(self) -> InterplanetaryDustModel:
-        """Returns the IPD model."""
-
-        return self._model
-
-    @property
-    def info(self) -> Optional[str]:
-        return self._model.meta.get("info")
-
-    @property
-    def observers(self) -> List[str]:
+    def supported_observers(self) -> List[str]:
         """Returns all observers suported by the ephemeridis."""
 
-        return list(solar_system_ephemeris.bodies) + ADDITIONAL_SUPPORTED_OBS
+        return list(solar_system_ephemeris.bodies) + ["l2 or semb-l2"]
 
     @quantity_input
     def get_emission(
         self,
         freq: Union[Quantity[u.Hz], Quantity[u.m]],
-        obs_time: Time,
-        obs: str = "earth",
         *,
+        obs: str = "earth",
+        obs_time: Time = Time.now(),
         obs_pos: Optional[Quantity[u.AU]] = None,
         pixels: Optional[Union[int, Sequence[int], NDArray[np.integer]]] = None,
         theta: Optional[Union[Quantity[u.rad], Quantity[u.deg]]] = None,
@@ -84,17 +70,16 @@ class Zodipy:
         binned: bool = False,
         return_comps: bool = False,
         coord_in: Literal["E", "G", "C"] = "E",
-        colorcorr_table: Optional[NDArray[np.floating]] = None,
     ) -> Quantity[u.MJy / u.sr]:
         """Returns simulated Zodiacal Emission.
 
         This function takes as arguments a frequency or wavelength (`freq`),
         time of observation (`obs_time`), an a Solar System observer (`obs`).
-        The position of the observer is computed using the pehemeris specified 
-        in the initialization of `Zodipy`. Optionally, the observer position 
-        can be explicitly specified with the `obs_pos` argument, which 
-        overrides`obs`. The pointing, for which to compute the emission, is 
-        specified either in the form of angles on the sky (`theta`, `phi`), or 
+        The position of the observer is computed using the pehemeris specified
+        in the initialization of `Zodipy`. Optionally, the observer position
+        can be explicitly specified with the `obs_pos` argument, which
+        overrides`obs`. The pointing, for which to compute the emission, is
+        specified either in the form of angles on the sky (`theta`, `phi`), or
         as HEALPIX pixels at some resolution (`pixels`, `nside`).
 
         Parameters
@@ -102,12 +87,13 @@ class Zodipy:
         freq
             Frequency or wavelength at which to evaluate the Zodiacal emission.
             Must have units compatible with Hz or m.
-        obs_time
-            Time of observation (`astropy.time.Time` object).
         obs
             The solar system observer. A list of all support observers (for a
             given ephemeridis) is specified in `observers` attribute of the
             `zodipy.Zodipy` instance. Defaults to 'earth'.
+        obs_time
+            Time of observation (`astropy.time.Time` object). Defaults to
+            current time.
         obs_pos
             The heliocentric ecliptic cartesian position of the observer in AU.
             Overrides the `obs` argument. Default is None.
@@ -136,11 +122,6 @@ class Zodipy:
         coord_in
             Coordinates frame of the pointing. Assumes 'E' (ecliptic coordinates)
             by default.
-        colorcorr_table
-            An array of shape (2, n) where the first column is temperatures
-            in K, and the second column the corresponding color corrections.
-            The color corrections should be for B(lambda, T) and is only applied
-            to the thermal contribution of the emission.
 
         Returns
         -------
@@ -151,7 +132,7 @@ class Zodipy:
             Zodiacal Emission timestream.
         """
 
-        if obs.lower() not in self.observers:
+        if obs.lower() not in self.supported_observers:
             raise ValueError(
                 f"observer {obs!r} not supported by ephemeridis "
                 f"{solar_system_ephemeris._value!r} or 'Zodipy'"
@@ -207,30 +188,18 @@ class Zodipy:
         if not isinstance(obs_time, Time):
             raise TypeError("argument 'obs_time' must be of type 'astropy.time.Time'")
 
-        obs_pos, earth_pos = _get_solar_system_bodies(obs, obs_time, obs_pos)
+        earth_position = get_earth_position(obs_time)
+        if obs_pos is None:
+            observer_position = get_observer_position(obs, obs_time, earth_position)
+        else:
+            observer_position = obs_pos.reshape(3,1)
 
-        # If the specific value of freq is not covered by the fitted spectral
-        # parameters in the model we interpolate/extrapolate to find a new
-        # emissivity, albedo and scattering phase coefficients.
-        freq = freq.to("GHz", equivalencies=u.spectral())
-        model = self._model
-        phase_coeffs = interp_phase_coeffs(
-            freq,
-            phase_coeffs=model.phase_coeffs,
-            phase_coeffs_spectrum=model.phase_coeffs_spectrum,
-        )
-        spectral_comp_parameters = interp_comp_spectral_params(
-            freq=freq,
-            emissivities=model.emissivities,
-            emissivity_spectrum=model.emissivity_spectrum,
-            albedos=model.albedos,
-            albedo_spectrum=model.albedo_spectrum,
-        )
+        frequency = freq.to("GHz", equivalencies=u.spectral())
 
         if binned:
             if pixels is not None:
                 unique_pixels, counts = np.unique(pixels, return_counts=True)
-                unit_vectors = _get_ecliptic_unit_vectors(
+                unit_vectors = get_unit_vector_from_pixels(
                     coord_in=coord_in,
                     pixels=unique_pixels,
                     nside=nside,
@@ -240,29 +209,45 @@ class Zodipy:
                     np.asarray([theta, phi]), return_counts=True, axis=1
                 )
                 unique_pixels = hp.ang2pix(nside, *unique_angles, lonlat=lonlat)
-                unit_vectors = _get_ecliptic_unit_vectors(
+                unit_vectors = get_unit_vector_from_angles(
                     coord_in=coord_in,
                     theta=unique_angles[0],
                     phi=unique_angles[1],
                     lonlat=lonlat,
                 )
 
-            emission = np.zeros((model.ncomps, hp.nside2npix(nside)))
-            for idx, (label, component) in enumerate(model.comps.items()):
-                emission[idx, unique_pixels] = trapezoidal(
-                    comp=component,
-                    freq=freq.value,
-                    line_of_sight=self._line_of_sights[label].value,
-                    observer_pos=obs_pos.value,
-                    earth_pos=earth_pos.value,
+            emission = np.zeros((self.model.n_components, hp.nside2npix(nside)))
+            for idx, (label, component) in enumerate(self.model.components.items()):
+                extrapolated_parameters = (
+                    self.model.get_extrapolated_component_parameters(label, frequency)
+                )
+                emissivity, albedo, phase_coefficients = extrapolated_parameters
+
+                # Create new callable with single argument as the distance along
+                # the line of sight (R)
+                emission_step_function = partial(
+                    get_emission_step,
+                    frequency=frequency.value,
+                    observer_position=observer_position.value,
+                    earth_position=earth_position.value,
                     unit_vectors=unit_vectors,
-                    cloud_offset=model.cloud_offset,
-                    T_0=model.T_0.value,
-                    delta=model.delta,
-                    emissivity=spectral_comp_parameters[label]["emissivity"],
-                    albedo=spectral_comp_parameters[label]["albedo"],
-                    phase_coeffs=phase_coeffs,
-                    colorcorr_table=colorcorr_table,
+                    component=component,
+                    T_0=self.model.T_0,
+                    delta=self.model.delta,
+                    emissivity=emissivity,
+                    albedo=albedo,
+                    phase_coefficients=phase_coefficients,
+                )
+                start, stop, n_steps = get_line_of_sight(
+                    component_label=label,
+                    observer_position=observer_position.value,
+                    unit_vectors=unit_vectors,
+                )
+                emission[idx, unique_pixels] = trapezoidal_regular_grid(
+                    get_emission_step=emission_step_function,
+                    start=start,
+                    stop=stop,
+                    n_steps=n_steps,
                 )
 
             emission[:, unique_pixels] *= counts
@@ -270,40 +255,57 @@ class Zodipy:
         else:
             if pixels is not None:
                 unique_pixels, indicies = np.unique(pixels, return_inverse=True)
-                unit_vectors = _get_ecliptic_unit_vectors(
+                unit_vectors = get_unit_vector_from_pixels(
                     coord_in=coord_in,
                     pixels=unique_pixels,
                     nside=nside,
                 )
-                emission = np.zeros((model.ncomps, len(pixels)))
+                emission = np.zeros((self.model.n_components, len(pixels)))
 
             else:
                 unique_angles, indicies = np.unique(
                     np.asarray([theta, phi]), return_inverse=True, axis=1
                 )
-                unit_vectors = _get_ecliptic_unit_vectors(
+                unit_vectors = get_unit_vector_from_angles(
                     coord_in=coord_in,
                     theta=unique_angles[0],
                     phi=unique_angles[1],
                     lonlat=lonlat,
                 )
-                emission = np.zeros((model.ncomps, len(theta)))
 
-            for idx, (label, component) in enumerate(model.comps.items()):
-                integrated_comp_emission = trapezoidal(
-                    comp=component,
-                    freq=freq.value,
-                    line_of_sight=self._line_of_sights[label].value,
-                    observer_pos=obs_pos.value,
-                    earth_pos=earth_pos.value,
+                emission = np.zeros((self.model.n_components, len(theta)))
+
+            for idx, (label, component) in enumerate(self.model.components.items()):
+                extrapolated_parameters = (
+                    self.model.get_extrapolated_component_parameters(label, frequency)
+                )
+                emissivity, albedo, phase_coefficients = extrapolated_parameters
+
+                # Create new callable with single argument as the distance along
+                # the line of sight (R)
+                emission_step_function = partial(
+                    get_emission_step,
+                    frequency=frequency.value,
+                    observer_position=observer_position.value,
+                    earth_position=earth_position.value,
                     unit_vectors=unit_vectors,
-                    cloud_offset=model.cloud_offset,
-                    T_0=model.T_0.value,
-                    delta=model.delta,
-                    emissivity=spectral_comp_parameters[label]["emissivity"],
-                    albedo=spectral_comp_parameters[label]["albedo"],
-                    phase_coeffs=phase_coeffs,
-                    colorcorr_table=colorcorr_table,
+                    component=component,
+                    T_0=self.model.T_0,
+                    delta=self.model.delta,
+                    emissivity=emissivity,
+                    albedo=albedo,
+                    phase_coefficients=phase_coefficients,
+                )
+                start, stop, n_steps = get_line_of_sight(
+                    component_label=label,
+                    observer_position=observer_position.value,
+                    unit_vectors=unit_vectors,
+                )
+                integrated_comp_emission = trapezoidal_regular_grid(
+                    get_emission_step=emission_step_function,
+                    start=start,
+                    stop=stop,
+                    n_steps=n_steps,
                 )
 
                 emission[idx] = integrated_comp_emission[indicies]
@@ -312,52 +314,3 @@ class Zodipy:
         emission = (emission << (u.W / u.Hz / u.m ** 2 / u.sr)).to(u.MJy / u.sr)
 
         return emission if return_comps else emission.sum(axis=0)
-
-
-def _get_solar_system_bodies(
-    obs: str, obs_time: Time, obs_pos: Optional[Quantity]
-) -> Tuple[Quantity[u.AU], Quantity[u.AU]]:
-    """Returns the observer and Earth's position."""
-
-    earth_skycoord = get_body("Earth", time=obs_time)
-    earth_skycoord = earth_skycoord.transform_to(HeliocentricMeanEcliptic)
-    earth_pos = earth_skycoord.represent_as("cartesian").xyz.to(u.AU)
-
-    if obs_pos is None:
-        if obs.lower() == "semb-l2":
-            # We assume that L2 is located at a constant distance from the
-            # Earth along Earths unit vector in heliocentric coordinates.
-            earth_length = np.linalg.norm(earth_pos)
-            earth_unit_vec = earth_pos / earth_length
-            semb_l2_length = earth_length + DISTANCE_EARTH_TO_L2
-            obs_pos_ = earth_unit_vec * semb_l2_length
-        else:
-            obs_skycoord = get_body(obs, time=obs_time)
-            obs_skycoord = obs_skycoord.transform_to(HeliocentricMeanEcliptic)
-            obs_pos_ = obs_skycoord.represent_as("cartesian").xyz.to(u.AU)
-    else:
-        obs_pos_ = obs_pos
-
-    return obs_pos_.reshape(3, 1), earth_pos.reshape(3, 1)
-
-
-def _get_ecliptic_unit_vectors(
-    coord_in: str,
-    pixels: Optional[Union[Sequence[int], NDArray[np.integer]]] = None,
-    nside: Optional[int] = None,
-    phi: Optional[Union[Quantity[u.rad], Quantity[u.deg]]] = None,
-    theta: Optional[Union[Quantity[u.rad], Quantity[u.deg]]] = None,
-    lonlat: bool = False,
-) -> NDArray[np.floating]:
-    """
-    Since the Interplanetary Dust Model is evaluated in Ecliptic coordinates,
-    we need to rotate any unit vectors (obtained from the pointing) defined in
-    another coordinate frame to ecliptic before evaluating the model.
-    """
-
-    if pixels is None:
-        unit_vectors = np.asarray(hp.ang2vec(theta, phi, lonlat=lonlat)).transpose()
-    else:
-        unit_vectors = np.asarray(hp.pix2vec(nside, pixels))
-
-    return np.asarray(hp.Rotator(coord=[coord_in, "E"])(unit_vectors))
